/**
 * Este programa é software livre; você pode redistribuí-lo e/ou modificá-lo sob
 * os termos da Licença Pública Geral GNU, conforme publicada pela Free Software
 * Foundation; tanto a versão 2 da Licença como (a seu critério) qualquer versão
 * mais nova.
 *
 * Este programa é distribuído na expectativa de ser útil, mas SEM QUALQUER
 * GARANTIA; sem mesmo a garantia implícita de COMERCIALIZAÇÃO ou de ADEQUAÇÃO A
 * QUALQUER PROPÓSITO EM PARTICULAR. Consulte a Licença Pública Geral GNU para
 * obter mais detalhes.
 *
 * Você deve ter recebido uma cópia da Licença Pública Geral GNU junto com este
 * programa; se não, escreva para a Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Prof. Emerson Ribeiro de Mello (2013) mello@ifsc.edu.br
 */
package util;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ImageIcon;
import javax.swing.Timer;
import jogo.Jogador;
import jogo.Principal;

/**
 *
 * @author mello
 */
public class JogoTimer extends Canvas implements ActionListener {

    private Principal framePrincipal; // classe com o frame principal
    private Timer timer; // Timer responsável pela atualização do jogo
    private int taxaDeAtualizacao; // a cada N milisegundos a tela é redesenhada
    private Jogador jog1;
    public int larguraBloco; // Dimensao dos personagens é 92 px
    private Image fundo;

    public JogoTimer(Principal pai) {
        super();
        this.framePrincipal = pai;
        this.setBackground(Color.WHITE);
        this.setIgnoreRepaint(true); // para melhorar a velocidade do redesenho
        this.taxaDeAtualizacao = 120;
        this.fundo = (new ImageIcon(this.getClass().getResource("/imagens/fundo.png"))).getImage();
    }

    /**
     * Método invocado para iniciar a partida
     */
    public void comecaOJogo() {
        // criando o Timer com taxa de atualização de N milisegundos
        this.timer = new Timer(taxaDeAtualizacao, this);
        // criando 2 buffer para acelerar a taxa de desenho
        createBufferStrategy(2);

        this.jog1 = new Jogador(this, "0.png", 1, 1);

        // Disparando o timer para começar a atualização do Loop principal do Jogo
        // Todo o conteúdo do método actionPerformed dessa classe será executado
        // a cada 60 milisegundos.
        this.timer.start();
    }

    /**
     * Principal METODO do JOGO, pois e' aqui que fica a logica que fara': -
     * processamento da logica do jogo (morreu, disparou, etc..) - atualizar as
     * coordenadas dos elementos na tela para o proximo redesenho - redesenhar
     * os elementos na tela
     *
     * Este método é invocado a cada N milisegundos pelo Timer
     *
     * @param e
     */
    @Override
    public void actionPerformed(ActionEvent ae) {
        this.framePrincipal.teclado.poll(); // processa os eventos no teclado
        this.processarLogica(); // atualiza as coordenadas dos elementos
        this.renderizar(); // desenha os elementos no buffer de desenho
        this.desenhar(); // faz a troca do buffer de desenho pelo buffer de apresentacao

    }

    private void processarLogica() {
        //Chama o atualizar dos elementos
        if (this.jog1 != null) {
            this.jog1.atualizar();
        }
    }

    /**
     * Renderiza no buffer secundário todos os elementos já com suas
     * coordenadas/estados atualizados.
     */
    public void renderizar() {
        Graphics g = getBufferStrategy().getDrawGraphics();
        Graphics2D g2 = (Graphics2D) g;
        //Limpando a tela

        // Gerando um gradiente que ficara' como cor de fundo
        //GradientPaint gp = new GradientPaint(25, 25, Color.white,50, 50, Color.RED, true);
        //g2.setPaint(gp);
        // ou pintando com uma cor definida por RGB
        Color c = new Color(255, 210, 174);
        g2.setColor(c);
        g2.fillRect(0, 0, getWidth(), getHeight());

        // desenhar cenario de fundo
        this.renderizarCenario(g2);

        // desenhando os elementos nao nulos
        if (this.jog1 != null) {
            this.jog1.desenhar(g2);
        }

        //liberando os contextos gráficos
        g.dispose();
        g2.dispose();

    }

    /**
     * Efetuando a troca das areas de desenho e apresentacao. No metodo
     * redenrizar os elemtos graficos foram desenhados em um buffer secundario,
     * agora iremos fazer com que esse buffer secundario torne-se primario, ou
     * seja, que apareca de fato na tela
     */
    public void desenhar() {
        if (!getBufferStrategy().contentsLost()) {
            getBufferStrategy().show();
        }
    }

    /**
     * Fim de jogo/rodada
     */
    public void finalizar() {
        this.jog1 = null;
        this.timer.stop();
    }

    /**
     * Desenha o cenário de fundo
     *
     * @param g contexto gráfico
     */
    public void renderizarCenario(Graphics2D g) {
        g.drawImage(this.fundo, 0, 0, this.framePrincipal);
    }
}
